/*
 * SPDX-FileCopyrightText: 2018-2023 Bruce Cowan <bruce@bcowan.me.uk>
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#pragma once

#include <stddef.h>

#include "types.h"

typedef struct _Array Array;

// Fundamental functions
Array * array_new         (FreeFunc    func);
Array * array_ref         (Array      *array);
void    array_unref       (Array      *array);

void    array_add         (Array      *array,
                           const void *val);
bool    array_remove_fast (Array      *array,
                           size_t      index);
bool    array_remove      (Array      *array,
                           size_t      index);
bool    array_set         (Array      *array,
                           size_t      index,
                           const void *val);

// Getters    
const void * array_get          (const Array *array,
                                 size_t       index);
size_t       array_get_length   (const Array *array);
size_t       array_get_capacity (const Array *array);

// Misc
void array_sort    (Array       *array,
                    CompareFunc  func);
void array_foreach (Array       *array,
                    ForEachFunc  func,
                    void        *user_data);
